#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "matrix.h"
#include "pnm.h"

#define N  7

typedef struct {
	int dx;
	int dy;
	float area;
} CENDATA;

/** Input: An image with square dimensions
	       Must be only one object inside the image and
		   must be too white (1) 
	Output: The centroid (dx,dy) and the object area
**/
void find_centroid( MATRIX *IMG, CENDATA *val ) 
{
	int i, j;
	float sumx, sumy, sumtot;
	byte **img;

	img = (byte **)IMG->ptr;

	sumx = sumy = sumtot = 0.0f;
	for( i=0; i<IMG->rows; i++ ) {
		for( j=0; j<IMG->cols; j++ ) {
			if ( img[i][j] ) {
				sumx += j;
				sumy += i;
				sumtot++;
			}
		}
	}

	val->dx = sumx/sumtot;
	val->dy = sumy/sumtot;
	val->area = sumtot;

	/**
	In my paper
	L. G. de la Fraga, J. M. Carazo and Gabor T. Herman,
	A Method for Obtaining a Low Resolution 3D Reconstruction
	from Very Noisy Projections,
	VII National Symposium on Pattern Recognition and Image Analysis,
	Computer Vision Center, Universitat Autnoma de Barcelona. Spain.
	23-25 April 1997. pp. 347-352.
	                                   
	Form factor = sqrt(u22/area)
	       u22  = central moment,2,2
	**/

}


int find_point_object( MATRIX *IN, int *x, int *y )
{
	int i, j;
	byte *p;

	for ( i=2; i<IN->rows-2; i++ ) {
		p = (byte *)IN->ptr[i];
		for ( j=2; j<IN->cols-2; j++ ) {
			if ( !p[j] ) {
				*x = j;
				*y = i;
				return 1; /** Sucess! **/
			} 
		}
	}
	return 0; /** Fault **/
}

int find_object ( char *name )
{
	MATRIX *IMG1, *OBJ;
	byte *pimg, *pout;
	int i, j, p,temp,counter;
	CENDATA cvals;
	char nameout[32];
	float cx, cy, d;

	printf("# Image_name dX  dY  |d|  Area  D_equivalent form_factor\n");

	/** Read the input image **/
	IMG1 = read_pnm( name );
	if ( IMG1 == NULL ){
		fprintf( stderr, "Can't read %s file\n", name );
		return( -1 );
	}
	/** limpio del borde de la imagen para evitar los bugs del
		programa **/
	pimg = (byte *) IMG1->ptr[0];
	for( j=0; j<IMG1->cols; j++ ) *pimg++ = 255;
	pimg = (byte *) IMG1->ptr[ IMG1->rows - 1 ];
	for( j=0; j<IMG1->cols; j++ ) *pimg++ = 255;

	for( i=1; i<IMG1->rows-1; i++ ){
		pimg = (byte *) IMG1->ptr[i];
	 	*pimg = 255;
		pimg = (byte *) IMG1->ptr[i] + IMG1->cols - 1;
	 	*pimg = 255;
	}
	/** termin de limpiar **/
	cx = (IMG1->cols - 1.0 )/2.0;
	cy = (IMG1->rows - 1.0 )/2.0;

	OBJ = matrix_allocate( IMG1->rows, IMG1->cols, 1 );
	if ( !OBJ ) {
		fprintf( stderr, "No memory for object image\n" );
		return( -1 );
	}

	counter=0;
	do {
		p = find_point_object( IMG1, &j, &i );
	
		/** printf("find_point_object -->encontro objeto en:j=%d, i=%d\n",j,i); **/
		if ( !p ) {
			fprintf( stderr, "Can't find any object point\n");
			break;
		}
		else {
			p = left ( IMG1, j, i );

			if ( p>=0 ) {   /** Succesful!! **/
		  		push( p, i );
			
				temp=do_fill(IMG1);
				/** Borramos la imagen de salida **/
				for( i=0; i<OBJ->rows; i++ ) {
					pimg = (byte *)OBJ->ptr[i];
					for( j=0; j<OBJ->cols; j++ ) *pimg++ = 0;
				}
				/** Generamos una imagen con el objeto **/			
				for( i=0; i<IMG1->rows; i++ ) {
					pimg = (byte *)IMG1->ptr[i];
					pout = (byte *)OBJ->ptr[i];
					for( j=0; j<IMG1->cols; j++ )
						if( pimg[j] == 10 ) pout[j] = 255;
				}
				
				sprintf( nameout, "s%04d", counter );
				printf("Grabando archivo ");
				write_pnm( OBJ, nameout );

				/** Borramos el objeto de la imagen original **/
				for( i=0; i<IMG1->rows; i++ ) {
					pimg = (byte *)IMG1->ptr[i];
					for( j=0; j<IMG1->cols; j++ )
						if( pimg[j] == 10 ) pimg[j] = 255;
				}
				
				find_centroid( OBJ, &cvals ); 

				/** Realiza la clasificacion **/
				d = fabs(929.167 - cvals.area) - fabs(735.75 - cvals.area);

				if ( d < 0 )
					printf( "Objeto %d es cuadrado\n", counter );
				else
					printf( "Objeto %d es elipse\n", counter );

				counter++;
			}
			else {
				clean_stack ( );
				fprintf( stderr, "There's not object!\n" );
			}
		}
	} while ( 1 );
	matrix_free( IMG1 ); 
	matrix_free( OBJ ); 
	return( 0 );
}


int main ( int argc, char *argv[] )
{
	char namein[64];

	if ( argc < 2 ) {
	 	printf("Args: img_name.pnm\n" ); 
		exit(1);
	}

	strncpy( namein, argv[1], 64 );

	if ( find_object ( namein ) == 0 ) printf("# OK!\n"); 
	else fprintf( stderr, "You have problems!\n" );

	return 0;
}


