#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "matrix.h"
#include "pnm.h"

#define N  7

typedef struct {
	float dx, dy;
	float area;
} CENDATA;

/** Input: An image with square dimensions
	       Must be only one object inside the image and
		   must be too white (1) 
	Output: The centroid (dx,dy) and the object area
**/
void find_centroid( MATRIX *IMG, CENDATA *val ) 
{
	int i, j;
	float sumx, sumy, sumtot;
	byte **img;

	img = (byte **)IMG->ptr;

	sumx = sumy = sumtot = 0.0f;
	for( i=0; i<IMG->rows; i++ ) {
		for( j=0; j<IMG->cols; j++ ) {
			if ( img[i][j] ) {
				sumx += j;
				sumy += i;
				sumtot++;
			}
		}
	}

	val->dx = sumx/sumtot;
	val->dy = sumy/sumtot;
	val->area = sumtot;

	/**
	In my paper
	L. G. de la Fraga, J. M. Carazo and Gabor T. Herman,
	A Method for Obtaining a Low Resolution 3D Reconstruction
	from Very Noisy Projections,
	VII National Symposium on Pattern Recognition and Image Analysis,
	Computer Vision Center, Universitat Autnoma de Barcelona. Spain.
	23-25 April 1997. pp. 347-352.
	                                   
	Form factor = sqrt(u22/area)
	       u22  = central moment,2,2
	**/

}

/** Calcula los momentos normalizados 

	Entrada: la imagen con el objeto. Los pixels del
             objeto tienen valor mayor que cero (0). El fondo es 0.
             Los valores de la estructura CENDATA *val ya se llenaron
             llamando a la funcin find_centroid()

	Salida: Un vector de tamao siete con los momentos normalizados.
	        La correspondencia para cada valor est en la funcin
		invariantes()
**/

double * momento_norm( MATRIX *IMG,  CENDATA *val ) 
{
	int i, j,k;
	float cx, cy, N00;
	double prod, mom_c, r;
	byte **img; 
	double *eta;
	int pn[N] = {0,0,1,1,2,2,3};
	int qn[N] = {2,3,1,2,0,1,0};
	
	eta = (double *)malloc(N*sizeof(double));
	
	img = (byte **)IMG->ptr;
	cx = val->dx;
	cy = val->dy;
	N00 = val->area;
	
	for(k=0; k<N; k++){
		/* Clculo de los momentos centrales */
		mom_c = 0.0f;
		for( i=0; i<IMG->rows; i++ ) {
			for( j=0; j<IMG->cols; j++ ) {
				if ( img[i][j] ) {
					prod = pow( j - cx, pn[k] ) * pow( i - cy, qn[k] );
					mom_c += prod;
				}
			}
		}
		r = (pn[k] + qn[k]) / 2.0 + 1.0;
		eta[k] = mom_c / pow( N00, r );
	}
	return eta;
}

/** Calcula los siete momentos invariantes

	Entrada: el vector de los momentos centrales normalizados generado
	         con la funcin momento_norm()
	Salida: Un vector con los momentos invariantes
**/
double * invariantes( double *eta )
{
	double *phi;
	double n02, n03, n11, n12, n20, n21, n30;
	
	phi = (double *)malloc(N*sizeof(double));
	
	n02 = eta[0];
	n03 = eta[1];
	n11 = eta[2];
	n12 = eta[3];
	n20 = eta[4];
	n21 = eta[5];
	n30 = eta[6];
	
	phi[0] = n20 + n02;
	phi[1] = (n20 - n02)*(n20 - n02) + 4*n11*n11;
	phi[2] = (n30 - 3.0*n12)*(n30 - 3.0*n12) + (3.0*n21-n03)*(3.0*n21-n03);
	phi[3] = (n30 + n12)*(n30 + n12) + (n21 + n03)*(n21 + n03);
	phi[4] = (n30-3.0*n12 )*(n30+n12 )*((n30+n12)*(n30+n12)
		-3.0*(n21+n03)*(n21+n03)) +
		(3.0*n21-n03 )*( n21+n03 )*( 3*(n30+n12)*(n30+n12)
		- (n21+n03)*(n21+n03) );
	phi[5] = (n20-n02)*((n30+n12)*(n30+n12) - (n21+n03)*(n21+n03))
		+ 4.0*n11*( n30+n12 )*( n21+n03 );
	phi[6] = ( 3.0*n21-n30 )*( n30+n12 )*( (n30+n12)*(n30+n12)
		- 3.0*(n21+n03)*(n21+n03) ) +
		( 3.0*n12-n30 )*( n21+n03 )*
		( 3.0*(n30+n12)*(n30+n12) - (n21+n03)*(n21+n03) );
	
	return phi;
}

int find_point_object( MATRIX *IN, int *x, int *y )
{
	int i, j;
	byte *p;

	for ( i=2; i<IN->rows-2; i++ ) {
		p = (byte *)IN->ptr[i];
		for ( j=2; j<IN->cols-2; j++ ) {
			if ( !p[j] ) {
				*x = j;
				*y = i;
				return 1; /** Sucess! **/
			} 
		}
	}
	return 0; /** Fault **/
}

int find_object ( char *name )
{
	MATRIX *IMG1, *OBJ;
	byte *pimg, *pout;
	int i, j, p,temp,counter;
	CENDATA cvals;
	float equidia, cx, cy;
	char nameout[32];
	double *etas, *phis;

	printf("# Image_name dX  dY  |d|  Area  D_equivalent form_factor\n");

	/** Read the input image **/
	IMG1 = read_pnm( name );
	if ( IMG1 == NULL ){
		fprintf( stderr, "Can't read %s file\n", name );
		return( -1 );
	}
	/** limpio del borde de la imagen para evitar los bugs del
		programa **/
	pimg = (byte *) IMG1->ptr[0];
	for( j=0; j<IMG1->cols; j++ ) *pimg++ = 255;
	pimg = (byte *) IMG1->ptr[ IMG1->rows - 1 ];
	for( j=0; j<IMG1->cols; j++ ) *pimg++ = 255;

	for( i=1; i<IMG1->rows-1; i++ ){
		pimg = (byte *) IMG1->ptr[i];
	 	*pimg = 255;
		pimg = (byte *) IMG1->ptr[i] + IMG1->cols - 1;
	 	*pimg = 255;
	}
	/** termin de limpiar **/
	cx = (IMG1->cols - 1.0 )/2.0;
	cy = (IMG1->rows - 1.0 )/2.0;

	OBJ = matrix_allocate( IMG1->rows, IMG1->cols, 1 );
	if ( !OBJ ) {
		fprintf( stderr, "No memory for object image\n" );
		return( -1 );
	}

	counter=0;
	do {
		p = find_point_object( IMG1, &j, &i );
	
		/* printf("find_point_object -->encontro objeto en:j=%d, i=%d\n",j,i); */
		if ( !p ) {
			fprintf( stderr, "Can't find any object point\n");
			break;
		}
		else {
			p = left ( IMG1, j, i );

			if ( p>=0 ) {   /** Succesful!! **/
		  		push( p, i );
			
				temp=do_fill(IMG1);
				/** Borramos la imagen de salida **/
				for( i=0; i<OBJ->rows; i++ ) {
					pimg = (byte *)OBJ->ptr[i];
					for( j=0; j<OBJ->cols; j++ ) *pimg++ = 0;
				}
				/** Generamos una imagen con el objeto **/			
				for( i=0; i<IMG1->rows; i++ ) {
					pimg = (byte *)IMG1->ptr[i];
					pout = (byte *)OBJ->ptr[i];
					for( j=0; j<IMG1->cols; j++ )
						if( pimg[j] == 10 ) pout[j] = 255;
				}
				sprintf( nameout, "out%04d", counter );
				/** printf("Grabando archivo "); **/
				write_pnm( OBJ, nameout );

				/** Borramos el objeto de la imagen original **/
				for( i=0; i<IMG1->rows; i++ ) {
					pimg = (byte *)IMG1->ptr[i];
					for( j=0; j<IMG1->cols; j++ )
						if( pimg[j] == 10 ) pimg[j] = 255;
				}
				
				find_centroid( OBJ, &cvals ); 
				etas = momento_norm( OBJ,  &cvals );
				phis = invariantes( etas );

				/**	For calculate centroid **/
				equidia = (float)sqrt( (4.0/M_PI)*cvals.area );

				/** Invariant moments 16/06/2005 **/
				printf(  "%s %f %f %f %f %f %f %f %f %f %f %f\n", nameout,
					cvals.dx, cvals.dy, cvals.area, equidia, 
					phis[0],phis[1],phis[2],phis[3],phis[4],phis[5],phis[6]);

				counter++;
				
			}
			else {
				clean_stack ( );
				fprintf( stderr, "There's not object!\n" );
			}
		}
	} while ( 1 );
	matrix_free( IMG1 ); 
	matrix_free( OBJ ); 
	return( 0 );
}


int main ( int argc, char *argv[] )
{
	char namein[64];

	if ( argc < 2 ) {
	 	printf("Args: img_name.pnm\n" ); 
		exit(1);
	}

	strncpy( namein, argv[1], 64 );

	if ( find_object ( namein ) == 0 ) printf("# OK!\n"); 
	else fprintf( stderr, "You have problems!\n" );

	return 0;
}


