/**
	Program to reduce to sixteen grey tones
	an image in 256 grey tones
	(it use histogram equalization technique)
	Input image must be in TIFF format

	Fraga, Jun 29, 2001

	Modification for method 1 : truncate to first 4 bytes
	Modification for method 2 : Improved Grey Scale
	Fraga, Jun 30, 2001 
	
**/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.h"
#include "utils.h"
#include "tiff.h"

MATRIX *improve_grey_scale( MATRIX *img )
{
	MATRIX *OUT;
	int i, j, k, l, s;
	int sum, win;
	byte *ptrin, *ptrout, *ptrcenter;
	unsigned int val;

	OUT = matrix_allocate( img->rows, img->cols, sizeof(byte) );
	if ( OUT==NULL ) {
		fprintf( stderr, "ERROR allocating memory for output image\n" );
		return NULL;
	}
	ptrin = (byte *) img->ptr[0];
	ptrout = (byte *) OUT->ptr[0];
	for ( j=0; j < img->cols; j++ ) *ptrout++ = (*ptrin++)>>4;

	ptrin = (byte *) img->ptr[img->rows-1];
	ptrout = (byte *) OUT->ptr[img->rows-1];
	for ( j=0; j < img->cols; j++ ) *ptrout++ = (*ptrin++)>>4;

	for ( i=1; i < img->rows; i++ ) {
		ptrin = (byte *) img->ptr[i];
		ptrout = (byte *) OUT->ptr[i];
		*ptrout = (*ptrin) >> 4;
		ptrout[img->cols-1] = ptrin[img->cols-1]>>4;
	}

	/** Let the neigbourhood be a square of odd size **/
	win = 3;
	k = win / 2;

	for ( i=0; i <= img->rows - win; i++ ) {
		ptrout = (byte *) OUT->ptr[i + k] + k;
		ptrcenter = (byte *) img->ptr[i + k] + k;

		for ( j=0; j <= img->cols - win; j++ ) {
			sum = 0;
			for ( l=0; l < win; l++ ) {
				ptrin = (byte *)img->ptr[ i + l ] + j;
				for ( s=0; s < win; s++ ) { 
					sum +=  *ptrin & 0x0f;
					ptrin++;
				}
			}
			sum -= ( *ptrcenter & 0x0f );
			sum &= 0x000f;
			if ( sum == 0x000f ) sum = 0x0000;
			val = *ptrcenter++ + sum;
			if ( val > 0x00ff ) val = 0x00ff;
			*ptrout++ = val >> 4;
		}
	}

	return OUT;	
}

MATRIX * truncated_bits( MATRIX * IN ) 
{
	MATRIX *OUT;
	int i, j;
	byte *pin, *pout, aux;

	OUT = matrix_allocate( IN->rows, IN->cols, 1 );
	if ( OUT == NULL ) {
		fprintf( stderr,"ERROR allocating memory for output equalized image\n");
		return NULL;
	}

	for ( i=0; i<IN->rows; i++ ) {
		pin = (byte *)IN->ptr[i];
		pout = (byte *)OUT->ptr[i];

		for ( j=0; j<IN->cols; j++ ) { 
			aux = pin[j] & 0xf0;
			pout[j] = aux >> 4; 			
		}
	}

	return OUT;
}

int do_work( char *namein, int howdo ) 
{
	MATRIX *IN, *EQ;
	FILEINFO finfo;
	int i, j;
	byte *iptr;
	

	IN = read_tiff( &finfo, namein ); 
	if ( IN==NULL ) {
		fprintf( stderr, "ERROR reading %s file\n", namein );
		return -1;
	}
	if ( IN->type != 5 ) {
		fprintf( stderr, "ERROR Bad format. Image must be in 256 grey tones\n");
		return -1;
	}
	if ( !howdo ){
		EQ = histogram_equalization( IN, 16 );
		if ( EQ == NULL ) {
			fprintf( stderr, "ERROR doing histogram equalization\n");
	 		return -1;
		}
	}
	else if ( howdo==1 ) {
		EQ = truncated_bits( IN );
	}
	else if ( howdo==2 ) {
		EQ = improve_grey_scale( IN );
	}

	EQ->type=2;
	// EQ->type=5;
	if ( write_tiff( EQ, "16tones.tif" ) != GOOD_WRITE )  
        return -1;

	matrix_free( IN );
	matrix_free( EQ );

	return 0;
}

int main( int argc, char *argv[] )
{
	char namein[32];
	int method = 0;

	if ( argc < 3 ) { 
		fprintf( stderr, "Use: %s image.tif method\n", argv[0] );
		fprintf( stderr, "\t method=0  histogram equalization\n", argv[0] );
		fprintf( stderr, "\t method=1  truncated bits\n", argv[0] );
		fprintf( stderr, "\t method=2  Improved Grey Scale (IGS)\n", argv[0] );
		return -1;
	}

	strncpy( namein, argv[1], 32 );
	method = atoi ( argv[2] );

	if ( do_work( namein, method ) >= 0 ) {
		fprintf( stderr, "OK!!!\n" );
		return 0;
	}
	else {
		fprintf( stderr, "ERROR: Something is wrong!!!\n" );
		exit(1);
	}
	return 0;
}
